package com.hamaf.hikyakucms.sample.schedule.ui;

import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import com.hamaf.hikyakucms.sample.schedule.dto.ScheduleCountDto;
import com.hamaf.hikyakucms.sample.schedule.process.HolidayProcess;
import com.hamaf.hikyakucms.sample.schedule.process.ScheduleProcess;
import com.hamaf.hikyakucms.sample.schedule.process.TargetProcess;
import com.hamaf.hikyaku.dataobject.CommonMessage;
import com.hamaf.hikyaku.security.SecurityCheckUtils;
import com.hamaf.hikyaku.security.dto.UserDto;
import com.hamaf.hikyaku.ui.ProcessBase;
import com.hamaf.hikyaku.util.DateUtil;
import com.hamaf.hikyaku.util.TextUtil;

/**
 * <p>^CgFGroupMouthList is .</p>
 * <p>F̃O[vXPW[̕\</p>
 * <br>NGXgp[^'day'̓t'offset','unit'̒lɑΉ
 * <br>t܂ވꂩ̃XPW[񋟂B
 * <br>'day'ꍇ́A{ΏۓƂ܂B
 * <br>'unit''M'(),'Y'(N)̉ꂩŎw肵܂B
 *
 * <p>쌠: Copyright (c) 2011/02/21</p>
 * @author Yoshinori
 * @version 1.0
 *
 */
public class GroupMouthList extends ProcessBase {

	private ScheduleProcess _sp = null;
	private TargetProcess _tp = null;
	private SecurityCheckUtils _scu = null;
	private HolidayProcess _hp = null;
	/**
	 * Creates a new <code>GroupMouthList.java</code> instance.
	 * <br>Default Constructor
	 */
	public GroupMouthList() {
		_sp = new ScheduleProcess();
		_tp = new TargetProcess();
		_scu = new SecurityCheckUtils();
		_hp = new HolidayProcess();
	}

	private final String REQ_DAY = "day";
  private final String REQ_OFF = "offset";
  private final String REQ_UNIT = "unit";
  private final String MSG_ID = "uid";
  private final String MSG_NM = "uname";
  private final String MSG_YM = "yyyymm";
  private final String MSG_DYM = "dispyymm";
  private final String MSG_CNT = "count";
	/* ( Javadoc)
	 * @see com.hamaf.hikyaku.ui.ProcessBase#exec(com.hamaf.hikyaku.dataobject.CommonMessage)
	 */
	@Override
	public CommonMessage exec(CommonMessage msg) throws Exception {
    String bday = _sp.compDay(msg.getReqStr(REQ_DAY), msg.getReqStr(REQ_OFF), msg.getReqStr(REQ_UNIT));
    bday = DateUtil.setDateTime(bday, Calendar.DATE, 1);
    String eday = DateUtil.addDateTime(bday, Calendar.MONTH, 1);
    eday = DateUtil.addDateTime(eday, Calendar.DATE, -1);
    int endday = Integer.parseInt(eday.substring(8));
    msg = setHeader(bday, endday, msg);

		List<String> uids = _tp.getUsers(getUserID(), sysId);
    for (String uid : uids) {
			msg = searchSchedule(uid, bday, eday, endday, msg);
		}

		return msg;
	}

	private CommonMessage setHeader(String sdate, int endday, CommonMessage msg) {
		Map<String, String> day = new HashMap<String, String>();
		day.put(MSG_NM, sdate.substring(0, 7));
		Map<String, String> yobi = new HashMap<String, String>();
		yobi.put(MSG_NM, "");
		Map<String, String> clr = new HashMap<String, String>();
		clr.put(MSG_NM, "#COLOR#");
		for (int i = 0; i <= endday; i++) {
			String si = String.valueOf(i);
			day.put(si, si);
			String yday = DateUtil.setDateTime(sdate, Calendar.DATE, i);
			String week = DateUtil.getYoubi(yday);
			String color = "#fffffff";
			if (_hp.isHoliday(yday, sysId)) color = "#ff99ff";
			yobi.put(si, week);
			clr.put(si, color);
		}
		msg.addResList(clr);
		msg.addResList(day);
		msg.addResList(yobi);
		msg.setResMsg(REQ_DAY, sdate);
		msg.setResMsg(REQ_OFF, "0");
		msg.setResMsg(REQ_UNIT, "M");
		msg.setResMsg(MSG_YM, sdate.substring(0, 8));
		msg.setResMsg(MSG_CNT, String.valueOf(endday));
		StringBuilder sb = new StringBuilder();
		sb.append(TextUtil.cnv2Zen(sdate.substring(0, 4)));
		sb.append(" N ");
		sb.append(TextUtil.cnv2Zen(sdate.substring(5, 7)));
		sb.append(" ");
		msg.setResMsg(MSG_DYM, sb.toString());
		return msg;
	}

	private CommonMessage searchSchedule(String uid, String sdate, String edate,
			int endday, CommonMessage msg) throws Exception {
		ScheduleCountDto scd = new ScheduleCountDto();
		scd.setRetrieveValue(ScheduleCountDto.USER_ID, uid);
		StringBuilder sb = new StringBuilder();
		sb.append("'");
		sb.append(sdate);
		sb.append("' AND '");
		sb.append(edate);
		sb.append("'");
		scd.setRetrieveValue(ScheduleCountDto.DAY, sb.toString());
		scd.setCondition(ScheduleCountDto.DAY, "BETWEEN");
		scd.setSysId(sysId);
		scd = _sp.find(scd);
		msg.addResList(setData(uid, endday, scd));
		return msg;
	}

	private Map<String, String> setData(String uid, int endday, ScheduleCountDto scd) throws Exception {
		Map<String, String> ret = new HashMap<String, String>();
		ret.put(MSG_ID, uid);
    UserDto ud = _scu.getUserInf(uid, getUserID());
    String rnm = ud.getItemValue(UserDto.ABBREV);
    if (rnm ==  null || rnm.length() == 0) {
    	ret.put(MSG_NM, ud.getItemValue(UserDto.NAME));
    } else {
    	ret.put(MSG_NM, rnm);
    }
		for (int i = 0; i <= endday; i++) {
			ret.put(String.valueOf(i), "");
		}
		for (List<String> key : scd) {
			int dd = Integer.parseInt(key.get(1).substring(8));
			ret.put(String.valueOf(dd), key.get(2));
		}
		return ret;
	}

}
